
#include "p3d.h"

Droplet::Droplet ( Hose *h, float *pos, float reset_height )
{
  z_reset = reset_height ;
  hose = h ;
  posn = pos ;
  sgSetVec3 ( speed, 0, 0, 0 ) ;
}


void Droplet::reset ()
{
  sgSetVec3 ( speed, SIDE_RANDOM * (float)(random()%1000-500)/500.f,
		     SIDE_RANDOM * (float)(random()%1000-500)/500.f,
		    -VERT_RANDOM * (float)(random()%1000)/1000.f ) ;
  sgAddVec3  ( speed, hose -> getSpeed() ) ;
  sgCopyVec3 ( posn , hose -> getPosn () ) ;
  posn [ 0 ] += (float)(random()%1000-500)/5000.f ;
  posn [ 1 ] += (float)(random()%1000-500)/5000.f ;
  posn [ 2 ] += (float)(random()%1000-500)/5000.f ;
}


void Hose::score ( sgVec3 pos )
{
  if ( game_state != GAME_RUNNING )
    return ;

  total_flow++ ;

  if ( fabs ( pos[0]-outlet[0] ) > 0.1 ||
       fabs ( pos[1]-outlet[1] ) > 0.1 )
    return ;

  total_score++ ;
}

void Hose::setOutlet ( sgVec3 pos )
{
  sgCopyVec3 ( outlet, pos ) ;

  for ( int i = 0 ; i < MAX_DROPLETS ; i++ )
    drop [ i ] -> setResetHeight ( pos[2] ) ;
}




Hose::Hose ()
{
  sgSetVec3 ( outlet, 0, 0, 0 ) ;
  sgSetVec3 ( speed , 0, 0, 0 ) ;
  sgSetVec3 ( posn  , 0, 0, 0 ) ;

  sgVec4 *clist = new sgVec4 [ 1 ] ;
  sgVec3 *nlist = new sgVec3 [ 1 ] ;
  sgVec2 *tlist = new sgVec2 [ 1 ] ;
  sgVec3 *vlist = new sgVec3 [ MAX_DROPLETS ] ;

  sgSetVec4 ( clist[0], 0.4, 1, 1, 1 ) ;
  sgSetVec3 ( nlist[0], 1, 0, 0 ) ;
  sgSetVec2 ( tlist[0], 0, 0 ) ;

  for ( int i = 0 ; i < MAX_DROPLETS ; i++ )
  {
    /*
      Scatter the drops all over so the bounding sphere is
      big.
    */

    sgSetVec3 ( vlist[i], (float) ( i - MAX_DROPLETS/2 ) * 10000.0f,
                          (float) ( i - MAX_DROPLETS/2 ) * 10000.0f,
                          (float) ( i - MAX_DROPLETS/2 ) * 10000.0f ) ;

    drop [ i ] = new Droplet ( this, vlist [ i ], -5.0f  ) ;
  }

  water = new ssgVTable ( (GLenum) GL_POINTS,
                    MAX_DROPLETS, vlist,
                         1      , nlist,
                         1      , tlist,
                         1      , clist ) ;

  water -> setState ( water_gst ) ;
  water -> getBSphere () ;

  for ( int j = 0 ; j < MAX_DROPLETS ; j++ )
  {
    int r = random() % 3000 ;

    drop [ j ] -> reset () ;

    for ( int k = 0 ; k < r ; k++ )
      drop [ k ] -> update ( 0.0 ) ;
  }
}


void Hose::update ()
{
  static double last = -1.0 ;
  double now = read_time_of_day() ;

  if ( last < 0.0 )
    last = now - 0.1 ;

  float dt = (float)( now - last ) ;
  last = now ;

  for ( int i = 0 ; i < MAX_DROPLETS ; i++ )
    drop [ i ] -> update ( dt ) ;
}


