

#include "p3d.h"

static int size[3] ;

Pipeline::Pipeline ()
{
  prev[0]=prev[1]=prev[2]=next[0]=next[1]=next[2]=0;
  connected = FALSE ;
  hose   = new Hose   ;
  inlet  = new Inlet  ;
  outlet = new Outlet ;
}

Puzzle::Puzzle ()
{ 
  size[0] = 5 ;
  size[1] = 5 ;
  size[2] = 5 ;
  
  sgSetCoord ( &pos, 0, 0, 0, 0, 0, 0 ) ;

  ground   = makeGround       () ;
  selector = makeBlueWireCube () ;

  spin = new ssgTransform ( &pos ) ;
  spin -> addKid ( ground ) ;

  for ( int i = 0 ; i < MAX_PIPELINE ; i++ )
  {
    pipeline[i] = new Pipeline ;

    spin -> addKid ( pipeline[i]->  hose -> getSSG () ) ;
    spin -> addKid ( pipeline[i]-> inlet -> getSSG () ) ;
    spin -> addKid ( pipeline[i]->outlet -> getSSG () ) ;
  }

  spin -> addKid ( selector ) ;

  sel [ 0 ] = 0 ;
  sel [ 1 ] = 0 ;
  sel [ 2 ] = 0 ;

  for ( int x = 0 ; x < size [ 0 ] ; x++ ) 
    for ( int y = 0 ; y < size [ 1 ] ; y++ ) 
      for ( int z = 0 ; z < size [ 2 ] ; z++ )
      {
        cell[x][y][z] = new Cell ( x, y, z, size[0] ) ;
        spin -> addKid ( cell[x][y][z]->getSSG() ) ;
      }

  reset () ;
}


void Puzzle::reset ()
{
  cursor ( level.inlet[0], level.inlet[1], 0 ) ;
  sgSetCoord ( &pos, 0, 0, 0, 0, 0, 0 ) ;

  sel [ 0 ] = 0 ;
  sel [ 1 ] = 0 ;
  sel [ 2 ] = 0 ;

  for ( int x = 0 ; x < size [ 0 ] ; x++ ) 
    for ( int y = 0 ; y < size [ 1 ] ; y++ ) 
      for ( int z = 0 ; z < size [ 2 ] ; z++ )
        cell[x][y][z]->set(WIRE_CELL) ; 

  for ( int i = 0 ; i < MAX_PIPELINE ; i++ )
  {
    pipeline[i]->prev [ 0 ] = pipeline[i]->next [ 0 ] = 0 ;
    pipeline[i]->prev [ 1 ] = pipeline[i]->next [ 1 ] = 0 ;
    pipeline[i]->prev [ 2 ] = pipeline[i]->next [ 2 ] = 0 ;
    pipeline[i]-> hose -> reset_score () ;

    /* If we ever have multiple inlets and outlets, we'll
      need a way to provide multiple coordinates */

    pipeline[i]->moveInlet  ( level. inlet[0], level. inlet[1] ) ;
    pipeline[i]->moveOutlet ( level.outlet[0], level.outlet[1] ) ;
  }

  accept () ;
}

void Pipeline::moveOutlet ( int x, int y )
{
  outlet_x = x ;
  outlet_y = y ;

  sgVec3 out ;

  sgSetVec3 ( out, (float)outlet_x + 0.5f - (float)size[0]/2.0f,
                   (float)outlet_y + 0.5f - (float)size[1]/2.0f,
                   - 0.2f - (float)size[2]/2.0f ) ;
  hose -> setOutlet ( out ) ;
  placeOutlet () ;
}

void Puzzle::update ()
{
  clamp ( sel  ) ;

  for ( int i = 0 ; i < MAX_PIPELINE ; i++ )
  {
    Pipeline *p = pipeline[i] ;
    clamp ( p->next ) ;
    clamp ( p->prev ) ;
    followHose ( p, p->inlet_x, p->inlet_y, size[2], PUZZ_DOWN ) ;
    p->placeInlet  () ;
    p->placeOutlet () ;
    p->hose->update () ;
    clamp ( p->next ) ;
    clamp ( p->prev ) ;
  }

  clamp ( sel ) ;

  static int flasher = 0 ;
  sgCoord spos ;
  sgSetVec3 ( spos.hpr, 0, 0, 0 ) ;

  flasher++ ;
  if ( flasher > 5 )
    sgSetVec3 ( spos.xyz, (float)sel[0] - (float)size[0]/2.0f + 0.51f,
                          (float)sel[1] - (float)size[1]/2.0f + 0.51f,
                          (float)sel[2] - (float)size[2]/2.0f + 0.51f ) ;
  else
    sgSetVec3 ( spos.xyz, 1000000, 1000000, 1000000 ) ;

  if ( flasher > 10 )
    flasher = 0 ;

  selector -> setTransform ( & spos ) ;
}


void clamp ( int *p )
{
  if ( p[0] < 0 ) p[0] = 0 ; if ( p[0] >= size[0] ) p[0] = size[0]-1 ;
  if ( p[1] < 0 ) p[1] = 0 ; if ( p[1] >= size[1] ) p[1] = size[1]-1 ;
  if ( p[2] < 0 ) p[2] = 0 ; if ( p[2] >= size[2] ) p[2] = size[2]-1 ;
}


void Pipeline::placeOutlet ()
{
  sgVec3 ipos ;

  sgSetVec3 ( ipos, (float)outlet_x + 0.5f - (float)size[0]/2.0f,
                    (float)outlet_y + 0.5f - (float)size[1]/2.0f,
                    - 0.5f - (float)size[2]/2.0f ) ;

  outlet -> put ( ipos ) ;
}


void Pipeline::placeInlet ()
{
  sgVec3 ipos ;

  sgSetVec3 ( ipos, (float)inlet_x + 0.5f - (float)size[0]/2.0f,
                    (float)inlet_y + 0.5f - (float)size[1]/2.0f,
                    (float)size[2] + 0.5f - (float)size[2]/2.0f ) ;

  inlet -> put ( ipos ) ;
}


void Puzzle::followHose ( Pipeline *p, int x, int y, int z, int dirn )
{
  p->connected = FALSE ;

  /*
    x,y,z is the address of the cell we are entering from
    and dirn is the direction the flow is travelling.
  */

  int nx = (dirn==PUZZ_LEFT) ? (x-1) : (dirn==PUZZ_RIGHT) ? (x+1) : x ;
  int ny = (dirn==PUZZ_OUT ) ? (y-1) : (dirn==PUZZ_IN   ) ? (y+1) : y ;
  int nz = (dirn==PUZZ_DOWN) ? (z-1) : (dirn==PUZZ_UP   ) ? (z+1) : z ;

  if ( nx == p->outlet_x && ny == p->outlet_y && nz < 0 )
    p->connected = TRUE ;

  /* Off edge of grid? */

  if ( nx < 0 || nx >= size[0] ||
       ny < 0 || ny >= size[1] ||
       nz < 0 || nz >= size[2] )
  {
    cursor ( x, y, z ) ;
    p->putHose ( x, y, z, dirn ) ;
    return ;
  }

  p->prev[0] = x ;
  p->prev[1] = y ;
  p->prev[2] = z ;

  int nd = cell[nx][ny][nz] -> flow ( dirn ) ;

  /* Flow into an empty cell */

  if ( nd < 0 )
  {
    cursor ( nx, ny, nz ) ;
    p -> putHose ( x, y, z, dirn ) ;
    return ;
  }
 
  followHose ( p, nx, ny, nz, nd ) ;
}


void Pipeline::putHose ( int x, int y, int z, int dirn )
{
  sgVec3 sp ;
  sgVec3 off ;

  switch ( dirn )
  {
    case PUZZ_UP   : sgSetVec3 ( sp , 0, 0, PRESSURE ) ;
                     sgSetVec3 ( off, 0, 0,   0.5    ) ; break ;
    case PUZZ_DOWN : sgSetVec3 ( sp , 0, 0,-PRESSURE ) ;
                     sgSetVec3 ( off, 0, 0,  -0.5    ) ; break ;
    case PUZZ_LEFT : sgSetVec3 ( sp ,-PRESSURE, 0, 0 ) ;
                     sgSetVec3 ( off,  -0.5   , 0, 0 ) ; break ;
    case PUZZ_RIGHT: sgSetVec3 ( sp , PRESSURE, 0, 0 ) ;
                     sgSetVec3 ( off,   0.5   , 0, 0 ) ; break ;
    case PUZZ_IN   : sgSetVec3 ( sp , 0, PRESSURE, 0 ) ;
                     sgSetVec3 ( off, 0,   0.5   , 0 ) ; break ;
    case PUZZ_OUT  : sgSetVec3 ( sp , 0,-PRESSURE, 0 ) ;
                     sgSetVec3 ( off, 0,  -0.5   , 0 ) ; break ;
  }

  hose -> setSpeed ( sp ) ;

  sgVec3 pos ;

  pos [ 0 ] = (float)x - (float)size[0]/2.0f + 0.5 ;
  pos [ 1 ] = (float)y - (float)size[1]/2.0f + 0.5 ;
  pos [ 2 ] = (float)z - (float)size[2]/2.0f + 0.5 ;

  sgAddVec3 ( pos, off ) ;

  hose -> setPosn ( pos ) ;
}



Puzzle::~Puzzle ()
{
  delete spin ;
}


