
#include "p3d.h"
#ifdef WIN32
#include <windows.h>
#include <time.h>
#else
#include <sys/time.h>
#include <time.h>
#include <unistd.h>
#endif
#include <stdarg.h>

#define MAX_STRING          30
#define MAX_STRING_LENGTH  256

int    displayed_score = 0 ;
double time_left = 0.0 ;
int    time_out  = FALSE ;

static fntRenderer *text = NULL ;

static double stop_time  = 0.0 ;

double read_time_of_day ()
{
#if defined( WIN32 ) && !defined( __CYGWIN32__ )
  _int64 u, v ;
  QueryPerformanceCounter   ((LARGE_INTEGER*) &u ) ;
  QueryPerformanceFrequency ((LARGE_INTEGER*) &v ) ;
  return (double)u / (double)v ;
#else
  timeval tv ;
  gettimeofday ( &tv, NULL ) ;

  return (double) tv . tv_sec + (double) tv . tv_usec / 1000000.0 ;
#endif
}



void start_stopwatch ()
{
  stop_time = read_time_of_day () + level.max_time ;
}



char debug_strings [ MAX_STRING ][ MAX_STRING_LENGTH ] ;
int next_string   = 0 ;
int stats_enabled = FALSE ;

void stToggle ()
{
  if ( stats_enabled )
    stats_enabled = FALSE ;
  else
  {
    stats_enabled = TRUE ;

    for ( int i = 0 ; i < MAX_STRING ; i++ )
       debug_strings [ i ][ 0 ] = '\0' ;

    next_string = 0 ;
  }
}


void stPrintf ( char *fmt, ... )
{
  char *p = debug_strings [ next_string++ ] ;

  if ( next_string >= MAX_STRING )
    next_string = 0 ;

  va_list ap ;
  va_start ( ap, fmt ) ;
/*
  Ideally, we should use this:

     vsnprintf ( p, MAX_STRING_LENGTH, fmt, ap ) ;

  ...but it's only in Linux   :-(
*/

  vsprintf ( p, fmt, ap ) ;

  va_end ( ap ) ;
}

static int    about_timer = 99999 ;
static int versions_timer = 99999 ;
static int  credits_timer = 99999 ;
static int    intro_timer =    0  ;
static int     help_timer = 99999 ;

void hide_status () { versions_timer = credits_timer =
                         intro_timer = help_timer = about_timer = 99999 ; }
void about       () {    about_timer = 0 ; }
void credits     () {  credits_timer = 0 ; }
void versions    () { versions_timer = 0 ; }
void help        () {     help_timer = 0 ; }

void drawText ( char *str, int sz, int x, int y )
{
  text -> setFont      ( font ) ;
  text -> setPointSize ( sz ) ;

  text -> begin () ;
    text -> start2f ( x, y ) ;
    text -> puts ( str ) ;
  text -> end () ;
}


void drawInverseDropShadowText ( char *str, int sz, int x, int y )
{
  glColor4f ( 1.0f, 1.0f, 1.0f, 1.0f ) ;
  drawText ( str, sz, x, y ) ;
  glColor4f ( 0.0f, 0.0f, 0.0f, 1.0f ) ;
  drawText ( str, sz, x+1, y+1 ) ;
}


void drawDropShadowText ( char *str, int sz, int x, int y )
{
  glColor4f ( 0.0f, 0.0f, 0.0f, 1.0f ) ;

  drawText ( str, sz, x, y ) ;

  glColor4f ( 1.0f, 1.0f, 1.0f, 1.0f ) ;

  drawText ( str, sz, x+1, y+1 ) ;
}


void drawHelpText ()
{
  drawDropShadowText ( "Press SPACE to toggle the menu.", 18, 70, 400 ) ;
  drawDropShadowText ( "Press ESCAPE to exit the game.", 18, 70, 370 ) ;
  drawDropShadowText ( "Use arrow keys and PgUp,PgDown,Home,End,Ins", 18, 70, 340 ) ;
  drawDropShadowText ( "          or Del to move the grid around.", 18, 70, 310 ) ;
  drawDropShadowText ( "Move the current pipe with A,S,Z,W,E,X", 18, 70, 280 ) ;
}


void drawTitleText ()
{
  drawDropShadowText ( "Totally Toobular", 20, 70, 400 ) ;
  drawDropShadowText ( "By Steve Baker", 12, 170, 385 ) ;
}


void drawIntroText ()
{
  drawTitleText () ;

  if ( intro_timer & 8 )
    drawDropShadowText ( "Press SPACE bar for menu.",
                       15, 10, 430 ) ;
}


char *aboutText [] =
{
  "  Yada, yada, yada.",
  NULL
} ;


void drawVersionsText ()
{
  char str [ 256 ] ;

#ifdef VERSION
  sprintf ( str, "Toobular: Version: %s", VERSION ) ;
#else
  sprintf ( str, "Toobular: Unknown Version." ) ;
#endif
  drawDropShadowText ( str, 15, 20, 250 ) ;

  sprintf ( str, "PLIB Version: %s", ssgGetVersion() ) ;
  drawDropShadowText ( str, 15, 20, 225 ) ;

  sprintf ( str, "OpenGL Version: %s", glGetString ( GL_VERSION ) ) ;
  drawDropShadowText ( str, 15, 20, 200 ) ;

  sprintf ( str, "OpenGL Vendor: %s", glGetString ( GL_VENDOR ) ) ;
  drawDropShadowText ( str, 15, 20, 175 ) ;

  sprintf ( str, "OpenGL Renderer: %s", glGetString ( GL_RENDERER ) ) ;

  if ( strlen ( str ) > 50 )
  {
    int l = strlen ( str ) ;
    int ll = 0 ;

    for ( int i = 0 ; i < l ; i++, ll++ )
    {
      if ( ll > 40 && str[i] == ' ' )
      {
        str[i] = '\n' ;
        ll = 0 ;
      }
    }
  }

  drawDropShadowText ( str, 15, 20, 150 ) ;

  if ( versions_timer & 8 )
    drawDropShadowText ( "Press SPACE to continue",
                       15, 10, 430 ) ;
}


void drawAboutText ()
{
  drawTitleText () ;

  drawDropShadowText ( "The History behind Toobular:",
                       20, 20, 250 ) ;

  for ( int i = 0 ; aboutText [ i ] != NULL ; i++ )
    drawDropShadowText ( aboutText [ i ],
                       10, 40, 230 - 10 * i ) ;

  if ( about_timer & 8 )
    drawDropShadowText ( "Press SPACE to continue",
                       15, 10, 430 ) ;
}


char *creditsText [] =
{
  "  Steve  Baker    - Coding, design, bug insertion.",
  "  Linus  Torvalds - For saving the world from the Tasteless Empire.",
  " ",
  "Special thanks to:",
  " ",
  "  Vasily Lewis, Curtis Olsen, Daryll Strauss, Brian Paul,",
  "  David Bucciarelli, Mark Kilgard",
  NULL
} ;



void drawCreditsText ()
{
  drawTitleText () ;

  drawDropShadowText ( "Credits:",
                       20, 70, 250 ) ;

  for ( int i = 0 ; creditsText [ i ] != NULL ; i++ )
    drawDropShadowText ( creditsText [ i ],
                       12, 100, 230 - 12 * i ) ;

  if ( credits_timer & 8 )
    drawDropShadowText ( "Press SPACE to continue",
                       15, 10, 430 ) ;
}


void drawTimer ()
{
  char str [ 256 ] ;

  if ( game_state == GAME_RUNNING )
    time_left = stop_time - read_time_of_day () ;

  if ( time_left > 0 )
  {
    int min     = (int) floor ( time_left / 60.0 ) ;
    int sec     = (int) floor ( time_left - (double) ( 60 * min ) ) ;
    int tenths  = (int) floor ( 10.0f * (time_left - (double)(sec + 60*min)));

    sprintf ( str, "%3d:%02d.%d", min,  sec,  tenths ) ;
    drawDropShadowText ( str, 18, 500, 430 ) ;
    time_out = FALSE ;
  }
  else
  {
    sprintf ( str, "Out of Time" ) ;
    drawDropShadowText ( str, 18, 500, 430 ) ;
    time_out = TRUE ;
  }
}

void drawScore ()
{
  if ( displayed_score > 0 )
  {
    char str [ 20 ] ;
    sprintf ( str, "%9d", displayed_score ) ;
    drawDropShadowText ( str, 18, 500, 410 ) ;
  }
}



void drawInGameScore ()
{
  char str [ 50 ] ;

  int spilled = puzzle->getFlow() - puzzle->getScore() ;

  sprintf ( str, "%4d Gallons Wasted = %5d", spilled,
			   (int)((float)spilled*level.flow_mult/1000.0f) ) ;
  drawDropShadowText ( str, 12, 400, 410 ) ;

  sprintf ( str, "%4d Pipes   = %5d", 0, (int)(0.0f * level.pipe_mult) ) ;
  drawDropShadowText ( str, 12, 400, 390 ) ;
}



void drawGameOverText ( int score )
{
  static int timer = 0 ;

  if ( time_left > 0 && puzzle -> isConnected () )
  {
    int final_score = (int)(( (float) puzzle -> getFlow() /
             (float)(level.max_time-time_left) ) * (float) time_left ) + score ;

    if ( final_score > displayed_score )
      displayed_score += (final_score - displayed_score) / 6 + 1 ;
  }
  else
    displayed_score += score ;

  drawTitleText () ;
  drawScore () ;
  drawTimer () ;

  glColor4f ( sin ( (float)timer/5.1f ) / 2.0f + 0.5f,
              sin ( (float)timer/6.3f ) / 2.0f + 0.5f,
              sin ( (float)timer/7.2f ) / 2.0f + 0.5f, 0.5 ) ;

  drawText ( "LEVEL"    , 50, 220, 230 ) ;
  drawText ( "COMPLETE!", 50, 150, 180 ) ;

  if ( timer++ & 8 )
  {
    drawDropShadowText ( "Press R to reset this level", 15, 10, 30 ) ;
    drawDropShadowText ( "Press S to start next level", 15, 10, 60 ) ;
  }
}


void drawGameIntroText ()
{
  static int timer = 0 ;

  if ( timer++ & 8 )
    drawDropShadowText ( "Press S to start", 15, 10, 430 ) ;

  if ( help_timer++ < 400 )
    drawHelpText () ;
  else
  if ( intro_timer++ < 400 )
    drawIntroText () ;
  else
  if ( credits_timer++ < 1600 )
    drawCreditsText () ;
  else
  if ( about_timer++ < 1600 )
    drawAboutText () ;
  else
  if ( versions_timer++ < 1600 )
    drawVersionsText () ;
}


void drawGameRunningText ( int score )
{
  displayed_score = score ;
  drawInGameScore () ;
  drawTimer () ;


  static time_t old_t = 0 ;
  static int nframes = 0 ;
  static int hz_rate = 0 ;
  time_t t = time ( NULL ) ;

  if ( t != old_t )
  {
    hz_rate = nframes / ( t - old_t ) ;
    nframes = 1 ;
    old_t = t ;
  }
  else
    nframes++ ;

  glColor4f ( 0.6, 0.0, 0.6, 1.0 ) ;

  if ( stats_enabled )
    for ( int i = 0 ; i < MAX_STRING ; i++ )
    {
      glRasterPos2f ( 10, 480 - 12 * (i + 2) ) ;

      int m = (i+next_string) % MAX_STRING ;
      char *s = debug_strings [ m ] ;

      for ( char *q = s ; *q != '\0' ; q++ )
	glutBitmapCharacter ( GLUT_BITMAP_9_BY_15, *q ) ;
    }

  if ( help_timer++ < 400 )
    drawHelpText () ;
  else
  if ( credits_timer++ < 1600 )
    drawCreditsText () ;
  else
  if ( about_timer++ < 1600 )
    drawAboutText () ;
  else
  if ( versions_timer++ < 1600 )
    drawVersionsText () ;
}



void drawStatusText ( int score )
{
  if ( text == NULL )
    text = new fntRenderer () ;

  glMatrixMode   ( GL_MODELVIEW ) ;
  glPushMatrix   () ;
  glLoadIdentity () ;

  glMatrixMode   ( GL_PROJECTION ) ;
  glPushMatrix   () ;
  glLoadIdentity () ;

  glPushAttrib   ( GL_ENABLE_BIT | GL_LIGHTING_BIT ) ;
  glDisable      ( GL_DEPTH_TEST ) ;
  glDisable      ( GL_LIGHTING   ) ;
  glDisable      ( GL_FOG        ) ;
  glDisable      ( GL_CULL_FACE  ) ;
  glDisable      ( GL_ALPHA_TEST ) ;
  glEnable       ( GL_BLEND      ) ;

  glOrtho        ( 0, 640, 0, 480, 0, 100 ) ;

  switch ( game_state )
  {
    case GAME_INTRO   : drawGameIntroText   ()        ; break ; 
    case GAME_DEBRIEF : drawGameOverText    ( score ) ; break ; 
    case GAME_RUNNING : drawGameRunningText ( score ) ; break ;
  }

  glPopAttrib  () ;
  glPopMatrix  () ;
  glMatrixMode ( GL_MODELVIEW ) ;
  glPopMatrix  () ;
}


