
#include "p3d.h"
#include <signal.h>

char  *toobular_datadir = NULL ;
GFX                *gfx = NULL ;
GUI                *gui = NULL ;
SoundSystem      *sound = NULL ;
ssgRoot          *scene = NULL ;
ssgTransform      *spin = NULL ;
Puzzle          *puzzle = NULL ;
float          spin_rot = 0.0f ;
float          spin_alt = 0.0f ;
float          spin_pit = 0.0f ;
int          game_state = GAME_START_OF_DAY ;

Level level ;

static void banner ()
{
  printf ( "\n\n" ) ;
  printf ( "   *TOTALLY* TUBULAR.\n" ) ;
  printf ( "               by Steve Baker\n" ) ;
  printf ( "                 <sjbaker1@airmail.net>\n" ) ;
  printf ( "                  http://toobular.sourceforge.net\n" ) ;
  printf ( "\n\n" ) ;
}

static void cmdline_help ()
{
  banner () ;

  printf ( "Usage:\n\n" ) ;
  printf ( "    toobular [OPTIONS]...\n\n" ) ;
  printf ( "Options:\n" ) ;
  printf ( "  -h, --help            Display this help message.\n" ) ;
  printf ( "  -D, --datadir DIR     Load the game data from DIR defaults\n" ) ;
  printf ( "                        to /usr/local/share/games/toobular\n" ) ;
  printf ( "\n" ) ;
}


int main ( int argc, char **argv )
{
  for ( int i = 1 ; i < argc ; i++ )
  {
    if ( argv[i][0] == '-' )
      switch ( argv[i][1] )
      {
        case '-' :
          {
            if ( strcmp ( & argv[i][2], "help" ) == 0 )
            {
              cmdline_help () ;
              exit ( 0 ) ;
            }
          }
          break ;

        case 'h' : case 'H' : cmdline_help () ; exit ( 0 ) ;
        default  : break ;
      }
  }

  /* Set tux_aqfh_datadir to the correct directory */

  if ( toobular_datadir == NULL )
  {
    if ( getenv ( "TOOBULAR_DATADIR" ) != NULL )
      toobular_datadir = getenv ( "TOOBULAR_DATADIR" ) ;
    else
    if ( access ( "data/levels.dat", F_OK ) == 0 )
      toobular_datadir = "." ;
    else
    if ( access ( "../data/levels.dat", F_OK ) == 0 )
      toobular_datadir = ".." ;
    else
#ifdef TOOBULAR_DATADIR
      toobular_datadir = TOOBULAR_DATADIR ;
#else
      toobular_datadir = "/usr/local/share/games/toobular" ;
#endif
  }

  fprintf ( stderr, "Data files will be fetched from: '%s'\n",
                                                    toobular_datadir ) ;

  if ( chdir ( toobular_datadir ) == -1 )
  {
    fprintf ( stderr, "Couldn't chdir() to '%s'.\n", toobular_datadir ) ;
    exit ( 1 ) ;
  }

  banner () ;

  level . max_time  =  200  ;
  level . pipe_mult =   0   ;
  level . flow_mult = 0.001 ;
  level . time_mult =   0   ;
  level . inlet  [ 0 ] = 0 ;
  level . inlet  [ 1 ] = 0 ;
  level . outlet [ 0 ] = 0 ;
  level . outlet [ 1 ] = 0 ;

  gfx   = new GFX ;

  sound = new SoundSystem ;
  sound -> change_track ( "mods/mfarmer.mod" ) ;

  gui   = new GUI ;

  scene = new ssgRoot ;
  spin  = new ssgTransform ;
  scene -> addKid ( spin ) ;

  puzzle = new Puzzle () ;

  spin -> addKid ( puzzle -> getSSG () ) ;
  spinLeft () ;

  signal ( 11, SIG_DFL ) ;
  glutMainLoop () ;
  return 0 ;
}



void spinPuzzle ()
{
  sgMat4 rot, pit, final ;

  sgMakeRotMat4 ( rot, spin_rot, 0, 0 ) ;
  sgMakeRotMat4 ( pit, 0, spin_pit, 0 ) ;
  sgMultMat4    ( final, pit, rot ) ;
  sgSetVec3     ( final[3], 0, 8, 0 /* spin_alt */ ) ;
  spin -> setTransform ( final ) ;
}

void spinUp ()
{
  spin_alt -= 0.4f ;
  spin_pit += 3.5f ;
  spinPuzzle () ;
}

void spinDown ()
{
  spin_alt += 0.4f ;
  spin_pit -= 3.5f ;
  spinPuzzle () ;
}

void spinRight ()
{
  spin_rot += 4.4f ;
  spinPuzzle () ;
}

void spinLeft ()
{
  spin_rot -= 4.4f ;
  spinPuzzle () ;
}


void nextLevel ()
{
  FILE *fd = fopen ( "data/levels.dat", "ra" ) ;

  if ( fd == NULL )
  {
    fprintf ( stderr, "Couldn't open data/levels.dat - Bad News!\n" ) ;
    exit ( -1 ) ;
  }

  /* Skip N records */

  char s [ 80 ] ;

  for ( int i = 0 ; i <= level.number && !feof(fd) ; i++ )
  {
    while ( fgets ( s, 80, fd ) != NULL )
      if ( s [ 0 ] == '{' )
        break ;
  }

  if ( feof ( fd ) )
  {
    /* No more levels! */

    fclose ( fd ) ;
    fprintf ( stderr, "Imagine a big fancy game completion sequence here!\n" ) ;
    return ;
  }

  fgets ( s, 80, fd ) ;

  sscanf ( s, " T=%f P=%f G=%f D=%f\n",
         & level.max_time,
         & level.pipe_mult,
         & level.flow_mult,
         & level.time_mult ) ;

  while ( fgets ( s, 80, fd ) != NULL )
  {
    char which ;
    int x, y ;

    if ( s [ 0 ] == '}' )
      break ;

    sscanf ( s, " %c=(%d,%d)\n", &which, &x, &y ) ;

    if ( which == 'I' )
    { level. inlet[0] = x ; level. inlet[1] = y ; }
    else
    { level.outlet[0] = x ; level.outlet[1] = y ; }
  }

  fprintf ( stderr, "\n\n" ) ;
  fprintf ( stderr, "Toobularity Level %2d\n", level.number ) ;
  fprintf ( stderr, "~~~~~~~~~~~~~~~~~~~~~\n\n" ) ;
  fprintf ( stderr, "Time Limit             = %f seconds\n", level.max_time ) ;
  fprintf ( stderr, "Points per gallon saved= %d\n", (int)(level.flow_mult) ) ;
  fprintf ( stderr, "Bonus per pipe segment = %d\n", (int)(level.pipe_mult) ) ;
  fprintf ( stderr, "Bonus per second saved = %d\n", (int)(level.time_mult) ) ;

  fprintf ( stderr, "Inlet  coords: (%d,%d)\n", level. inlet[0],
                                                level. inlet[1] ) ;
  fprintf ( stderr, "Outlet coords: (%d,%d)\n", level.outlet[0],
                                                level.outlet[1] ) ;
  fprintf ( stderr, "\n\n" ) ;

  level.intro = "Hello World" ;
}


void startLevel ()
{
  game_state = GAME_RUNNING ;
  start_stopwatch () ;
  puzzle -> reset () ;
  puzzle -> update () ;
  puzzle -> accept () ;
  sound  -> playSfx ( SOUND_CLAP ) ;
}

void plumMainLoop ()
{
  switch ( game_state )
  {
    case GAME_START_OF_DAY : level.number = 0 ;
                             nextLevel () ;
                             startLevel () ; break ;
    case GAME_INTRO        : startLevel () ; break ;
    case GAME_DEBRIEF      : break ;

    case GAME_RUNNING :
      if ( time_out || puzzle -> isConnected () )
        game_state = GAME_DEBRIEF ;
      break ;
  }

  puzzle -> update () ;
  gfx    -> update () ;
  gui    -> update () ;
  sound  -> update () ;
  gfx    -> done   () ;  /* Swap buffers! */
}


